@@ -31,7 +31,7 @@ output = table
 from __future__ import annotations
 
 import re
-from colorsys import hls_to_rgb, rgb_to_hls
+from colorsys import hls_to_rgb, hsv_to_rgb, rgb_to_hls, rgb_to_hsv
 from functools import lru_cache
 from operator import itemgetter
 from typing import Callable, NamedTuple
@@ -53,7 +53,7 @@ _TRUECOLOR = ColorType.TRUECOLOR
 
 
 class HSL(NamedTuple):
-    """A color in HLS (Hue, Saturation, Lightness) format."""
+    """A color in HSL (Hue, Saturation, Lightness) format."""
 
     h: float
     """Hue in range 0 to 1."""
@@ -82,7 +82,7 @@ class HSV(NamedTuple):
     s: float
     """Saturation in range 0 to 1."""
     v: float
-    """Value un range 0 to 1."""
+    """Value in range 0 to 1."""
 
 
 class Lab(NamedTuple):
@@ -110,11 +110,11 @@ hsla{OPEN_BRACE}({DECIMAL}{COMMA}{PERCENT}{COMMA}{PERCENT}{COMMA}{DECIMAL}){CLOS
     re.VERBOSE,
 )
 
-# Fast way to split a string of 6 characters in to 3 pairs of 2 characters
+# Fast way to split a string of 6 characters into 3 pairs of 2 characters
 _split_pairs3: Callable[[str], tuple[str, str, str]] = itemgetter(
     slice(0, 2), slice(2, 4), slice(4, 6)
 )
-# Fast way to split a string of 8 characters in to 4 pairs of 2 characters
+# Fast way to split a string of 8 characters into 4 pairs of 2 characters
 _split_pairs4: Callable[[str], tuple[str, str, str, str]] = itemgetter(
     slice(0, 2), slice(2, 4), slice(4, 6), slice(6, 8)
 )
@@ -177,6 +177,7 @@ class Color(NamedTuple):
         return cls(0, 0, 0, alpha_percentage / 100.0, auto=True)
 
     @classmethod
+    @lru_cache(maxsize=1024)
     def from_rich_color(
         cls, rich_color: RichColor | None, theme: TerminalTheme | None = None
     ) -> Color:
@@ -192,16 +193,18 @@ class Color(NamedTuple):
         if rich_color is None:
             return TRANSPARENT
         r, g, b = rich_color.get_truecolor(theme)
-        return cls(r, g, b)
+        return cls(
+            r, g, b, ansi=rich_color.number if rich_color.is_system_defined else None
+        )
 
     @classmethod
     def from_hsl(cls, h: float, s: float, l: float) -> Color:
-        """Create a color from HLS components.
+        """Create a color from HSL components.
 
         Args:
             h: Hue.
-            l: Lightness.
             s: Saturation.
+            l: Lightness.
 
         Returns:
             A new color.
@@ -209,6 +212,21 @@ class Color(NamedTuple):
         r, g, b = hls_to_rgb(h, l, s)
         return cls(int(r * 255 + 0.5), int(g * 255 + 0.5), int(b * 255 + 0.5))
 
+    @classmethod
+    def from_hsv(cls, h: float, s: float, v: float) -> Color:
+        """Create a color from HSV components.
+
+        Args:
+            h: Hue.
+            s: Saturation.
+            v: Value.
+
+        Returns:
+            A new color.
+        """
+        r, g, b = hsv_to_rgb(h, s, v)
+        return cls(int(r * 255 + 0.5), int(g * 255 + 0.5), int(b * 255 + 0.5))
+
     @property
     def inverse(self) -> Color:
         """The inverse of this color.
@@ -247,7 +265,7 @@ class Color(NamedTuple):
         Returns:
             A color object as used by Rich.
         """
-        r, g, b, _a, ansi, _ = self
+        r, g, b, a, ansi, _ = self
         if ansi is not None:
             return RichColor.parse("default") if ansi < 0 else RichColor.from_ansi(ansi)
         return RichColor(
@@ -283,6 +301,19 @@ class Color(NamedTuple):
         h, l, s = rgb_to_hls(r, g, b)
         return HSL(h, s, l)
 
+    @property
+    def hsv(self) -> HSV:
+        """This color in HSV format.
+
+        HSV color is an alternative way of representing a color, which can be used in certain color calculations.
+
+        Returns:
+            Color encoded in HSV format.
+        """
+        r, g, b = self.normalized
+        h, s, v = rgb_to_hsv(r, g, b)
+        return HSV(h, s, v)
+
     @property
     def brightness(self) -> float:
         """The human perceptual brightness.
@@ -298,7 +329,7 @@ class Color(NamedTuple):
     def hex(self) -> str:
         """The color in CSS hex form, with 6 digits for RGB, and 8 digits for RGBA.
 
-        For example, `"#46b3de"` for an RGB color, or `"#3342457f"` for a color with alpha.
+        For example, `"#46B3DE"` for an RGB color, or `"#3342457F"` for a color with alpha.
         """
         r, g, b, a, ansi, _ = self.clamped
         if ansi is not None:
@@ -313,7 +344,7 @@ class Color(NamedTuple):
     def hex6(self) -> str:
         """The color in CSS hex form, with 6 digits for RGB. Alpha is ignored.
 
-        For example, `"#46b3de"`.
+        For example, `"#46B3DE"`.
         """
         r, g, b, _a, _, _ = self.clamped
         return f"#{r:02X}{g:02X}{b:02X}"
@@ -327,6 +358,8 @@ class Color(NamedTuple):
         r, g, b, a, ansi, auto = self
         if auto:
             alpha_percentage = clamp(a, 0.0, 1.0) * 100.0
+            if alpha_percentage == 100:
+                return "auto"
             if not alpha_percentage % 1:
                 return f"auto {int(alpha_percentage)}%"
             return f"auto {alpha_percentage:.1f}%"
@@ -377,8 +410,8 @@ class Color(NamedTuple):
         """
         if self.ansi is not None:
             return self
-        r, g, b, a, _, _ = self
-        return Color(r, g, b, a * alpha)
+        r, g, b, a, _ansi, auto = self
+        return Color(r, g, b, a * alpha, auto=auto)
 
     @lru_cache(maxsize=1024)
     def blend(
@@ -450,6 +483,15 @@ class Color(NamedTuple):
     def __add__(self, other: object) -> Color:
         if isinstance(other, Color):
             return self.blend(other, other.a, 1.0)
+        elif other is None:
+            return self
+        return NotImplemented
+
+    def __radd__(self, other: object) -> Color:
+        if isinstance(other, Color):
+            return self.blend(other, other.a, 1.0)
+        elif other is None:
+            return self
         return NotImplemented
 
     @classmethod
